@@ -1,34 +1,5 @@
-/*-
- *   BSD LICENSE
- *
- *   Copyright(c) 2010-2014 Intel Corporation. All rights reserved.
- *   All rights reserved.
- *
- *   Redistribution and use in source and binary forms, with or without
- *   modification, are permitted provided that the following conditions
- *   are met:
- *
- *     * Redistributions of source code must retain the above copyright
- *       notice, this list of conditions and the following disclaimer.
- *     * Redistributions in binary form must reproduce the above copyright
- *       notice, this list of conditions and the following disclaimer in
- *       the documentation and/or other materials provided with the
- *       distribution.
- *     * Neither the name of Intel Corporation nor the names of its
- *       contributors may be used to endorse or promote products derived
- *       from this software without specific prior written permission.
- *
- *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
- *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
- *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
- *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
- *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
- *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* SPDX-License-Identifier: BSD-3-Clause
+ * Copyright(c) 2010-2014 Intel Corporation
  */
 
 #ifndef TEST_ACL_H_
@@ -111,13 +82,13 @@ struct rte_acl_ipv4vlan_rule invalid_layout_rules[] = {
 		{
 				.data = {.userdata = 1, .category_mask = 1,
 					.priority = 1},
-				.src_addr = IPv4(10,0,0,0),
+				.src_addr = RTE_IPV4(10,0,0,0),
 				.src_mask_len = 24,
 		},
 		{
 				.data = {.userdata = 2, .category_mask = 1,
 					.priority = 1},
-				.dst_addr = IPv4(10,0,0,0),
+				.dst_addr = RTE_IPV4(10,0,0,0),
 				.dst_mask_len = 24,
 		},
 		/* test src and dst ports */
@@ -153,8 +124,8 @@ struct rte_acl_ipv4vlan_rule invalid_layout_rules[] = {
  * results using the wrong data layout.
  */
 struct ipv4_7tuple invalid_layout_data[] = {
-		{.ip_src = IPv4(10,0,1,0)},             /* should not match */
-		{.ip_src = IPv4(10,0,0,1), .allow = 2}, /* should match 2 */
+		{.ip_src = RTE_IPV4(10,0,1,0)},             /* should not match */
+		{.ip_src = RTE_IPV4(10,0,0,1), .allow = 2}, /* should match 2 */
 		{.port_src = 100, .allow = 4},          /* should match 4 */
 		{.port_dst = 0xf, .allow = 6},          /* should match 6 */
 };
@@ -171,7 +142,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 1, .category_mask = ACL_ALLOW_MASK,
 						.priority = 230},
-				.dst_addr = IPv4(192,168,0,0),
+				.dst_addr = RTE_IPV4(192,168,0,0),
 				.dst_mask_len = 16,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -182,7 +153,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 2, .category_mask = ACL_ALLOW_MASK,
 						.priority = 330},
-				.dst_addr = IPv4(192,168,1,0),
+				.dst_addr = RTE_IPV4(192,168,1,0),
 				.dst_mask_len = 24,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -193,7 +164,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 3, .category_mask = ACL_DENY_MASK,
 						.priority = 230},
-				.dst_addr = IPv4(192,168,1,50),
+				.dst_addr = RTE_IPV4(192,168,1,50),
 				.dst_mask_len = 32,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -206,7 +177,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 4, .category_mask = ACL_ALLOW_MASK,
 						.priority = 240},
-				.src_addr = IPv4(10,0,0,0),
+				.src_addr = RTE_IPV4(10,0,0,0),
 				.src_mask_len = 8,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -217,7 +188,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 5, .category_mask = ACL_ALLOW_MASK,
 						.priority = 340},
-				.src_addr = IPv4(10,1,1,0),
+				.src_addr = RTE_IPV4(10,1,1,0),
 				.src_mask_len = 24,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -228,7 +199,7 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 6, .category_mask = ACL_DENY_MASK,
 						.priority = 240},
-				.src_addr = IPv4(10,1,1,1),
+				.src_addr = RTE_IPV4(10,1,1,1),
 				.src_mask_len = 32,
 				.src_port_low = 0,
 				.src_port_high = 0xffff,
@@ -422,9 +393,9 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 				.data = {.userdata = 24, .category_mask = ACL_ALLOW_MASK,
 						.priority = 400},
 				/** make sure that unmasked bytes don't fail! */
-				.dst_addr = IPv4(1,2,3,4),
+				.dst_addr = RTE_IPV4(1,2,3,4),
 				.dst_mask_len = 16,
-				.src_addr = IPv4(5,6,7,8),
+				.src_addr = RTE_IPV4(5,6,7,8),
 				.src_mask_len = 24,
 				.proto = 0x5,
 				.proto_mask = 0xff,
@@ -440,9 +411,9 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 25, .category_mask = ACL_DENY_MASK,
 						.priority = 400},
-				.dst_addr = IPv4(5,6,7,8),
+				.dst_addr = RTE_IPV4(5,6,7,8),
 				.dst_mask_len = 24,
-				.src_addr = IPv4(1,2,3,4),
+				.src_addr = RTE_IPV4(1,2,3,4),
 				.src_mask_len = 16,
 				.proto = 0x5,
 				.proto_mask = 0xff,
@@ -458,9 +429,9 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 26, .category_mask = ACL_ALLOW_MASK,
 						.priority = 500},
-				.dst_addr = IPv4(1,2,3,4),
+				.dst_addr = RTE_IPV4(1,2,3,4),
 				.dst_mask_len = 8,
-				.src_addr = IPv4(5,6,7,8),
+				.src_addr = RTE_IPV4(5,6,7,8),
 				.src_mask_len = 32,
 				.proto = 0x5,
 				.proto_mask = 0xff,
@@ -474,9 +445,9 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 		{
 				.data = {.userdata = 27, .category_mask = ACL_DENY_MASK,
 						.priority = 500},
-				.dst_addr = IPv4(5,6,7,8),
+				.dst_addr = RTE_IPV4(5,6,7,8),
 				.dst_mask_len = 32,
-				.src_addr = IPv4(1,2,3,4),
+				.src_addr = RTE_IPV4(1,2,3,4),
 				.src_mask_len = 8,
 				.proto = 0x5,
 				.proto_mask = 0xff,
@@ -492,20 +463,20 @@ struct rte_acl_ipv4vlan_rule acl_test_rules[] = {
 /* data for ACL unit test */
 struct ipv4_7tuple acl_test_data[] = {
 /* testing single rule aspects */
-		{.ip_src = IPv4(10,0,0,0), .allow = 4}, /* should match 4 */
-		{.ip_src = IPv4(10,1,1,2), .allow = 5}, /* should match 5 */
-		{.ip_src = IPv4(10,1,1,1), .allow = 5,
+		{.ip_src = RTE_IPV4(10,0,0,0), .allow = 4}, /* should match 4 */
+		{.ip_src = RTE_IPV4(10,1,1,2), .allow = 5}, /* should match 5 */
+		{.ip_src = RTE_IPV4(10,1,1,1), .allow = 5,
 				.deny = 6},                     /* should match 5, 6 */
-		{.ip_dst = IPv4(10,0,0,0)},             /* should not match */
-		{.ip_dst = IPv4(10,1,1,2)},             /* should not match */
-		{.ip_dst = IPv4(10,1,1,1)},             /* should not match */
+		{.ip_dst = RTE_IPV4(10,0,0,0)},             /* should not match */
+		{.ip_dst = RTE_IPV4(10,1,1,2)},             /* should not match */
+		{.ip_dst = RTE_IPV4(10,1,1,1)},             /* should not match */
 
-		{.ip_src = IPv4(192,168,2,50)},             /* should not match */
-		{.ip_src = IPv4(192,168,1,2)},              /* should not match */
-		{.ip_src = IPv4(192,168,1,50)},             /* should not match */
-		{.ip_dst = IPv4(192,168,2,50), .allow = 1}, /* should match 1 */
-		{.ip_dst = IPv4(192,168,1,49), .allow = 2}, /* should match 2 */
-		{.ip_dst = IPv4(192,168,1,50), .allow = 2,
+		{.ip_src = RTE_IPV4(192,168,2,50)},             /* should not match */
+		{.ip_src = RTE_IPV4(192,168,1,2)},              /* should not match */
+		{.ip_src = RTE_IPV4(192,168,1,50)},             /* should not match */
+		{.ip_dst = RTE_IPV4(192,168,2,50), .allow = 1}, /* should match 1 */
+		{.ip_dst = RTE_IPV4(192,168,1,49), .allow = 2}, /* should match 2 */
+		{.ip_dst = RTE_IPV4(192,168,1,50), .allow = 2,
 				.deny = 3},                         /* should match 2, 3 */
 
 		{.vlan = 0x64, .allow = 7},            /* should match 7 */
@@ -544,20 +515,20 @@ struct ipv4_7tuple acl_test_data[] = {
 		{.proto = 0x5, .allow = 22, .deny = 23},  /* should match 22, 23 */
 
 /* testing matching multiple rules at once */
-		{.vlan = 0x5, .ip_src = IPv4(10,1,1,1),
+		{.vlan = 0x5, .ip_src = RTE_IPV4(10,1,1,1),
 				.allow = 5, .deny = 9},               /* should match 5, 9 */
-		{.vlan = 0x5, .ip_src = IPv4(192,168,2,50),
+		{.vlan = 0x5, .ip_src = RTE_IPV4(192,168,2,50),
 				.allow = 8, .deny = 9},               /* should match 8, 9 */
-		{.vlan = 0x55, .ip_src = IPv4(192,168,1,49),
+		{.vlan = 0x55, .ip_src = RTE_IPV4(192,168,1,49),
 				.allow = 8},                          /* should match 8 */
 		{.port_dst = 80, .port_src = 1024,
 				.allow = 13, .deny = 20},             /* should match 13,20 */
 		{.port_dst = 79, .port_src = 1024,
 				.allow = 14, .deny = 20},             /* should match 14,20 */
-		{.proto = 0x5, .ip_dst = IPv4(192,168,2,50),
+		{.proto = 0x5, .ip_dst = RTE_IPV4(192,168,2,50),
 				.allow = 1, .deny = 23},               /* should match 1, 23 */
 
-		{.proto = 0x5, .ip_dst = IPv4(192,168,1,50),
+		{.proto = 0x5, .ip_dst = RTE_IPV4(192,168,1,50),
 				.allow = 2, .deny = 23},              /* should match 2, 23 */
 		{.vlan = 0x64, .domain = 0x5,
 				.allow = 11, .deny = 12},             /* should match 11, 12 */
@@ -566,16 +537,16 @@ struct ipv4_7tuple acl_test_data[] = {
 		{.proto = 0x5, .port_dst = 80,
 				.allow = 13, .deny = 23},             /* should match 13, 23 */
 		{.proto = 0x51, .port_src = 5000},            /* should not match */
-		{.ip_src = IPv4(192,168,1,50),
-				.ip_dst = IPv4(10,0,0,0),
+		{.ip_src = RTE_IPV4(192,168,1,50),
+				.ip_dst = RTE_IPV4(10,0,0,0),
 				.proto = 0x51,
 				.port_src = 5000,
 				.port_dst = 5000},                    /* should not match */
 
 /* test full packet rules */
 		{
-				.ip_dst = IPv4(1,2,100,200),
-				.ip_src = IPv4(5,6,7,254),
+				.ip_dst = RTE_IPV4(1,2,100,200),
+				.ip_src = RTE_IPV4(5,6,7,254),
 				.proto = 0x5,
 				.vlan = 0x8100,
 				.domain = 0x64,
@@ -585,8 +556,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.deny = 23
 		}, /* should match 23, 24 */
 		{
-				.ip_dst = IPv4(5,6,7,254),
-				.ip_src = IPv4(1,2,100,200),
+				.ip_dst = RTE_IPV4(5,6,7,254),
+				.ip_src = RTE_IPV4(1,2,100,200),
 				.proto = 0x5,
 				.vlan = 0x8100,
 				.domain = 0x64,
@@ -596,8 +567,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.deny = 25
 		}, /* should match 13, 25 */
 		{
-				.ip_dst = IPv4(1,10,20,30),
-				.ip_src = IPv4(5,6,7,8),
+				.ip_dst = RTE_IPV4(1,10,20,30),
+				.ip_src = RTE_IPV4(5,6,7,8),
 				.proto = 0x5,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -606,8 +577,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.deny = 23
 		}, /* should match 23, 26 */
 		{
-				.ip_dst = IPv4(5,6,7,8),
-				.ip_src = IPv4(1,10,20,30),
+				.ip_dst = RTE_IPV4(5,6,7,8),
+				.ip_src = RTE_IPV4(1,10,20,30),
 				.proto = 0x5,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -616,8 +587,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.deny = 27
 		}, /* should match 13, 27 */
 		{
-				.ip_dst = IPv4(2,2,3,4),
-				.ip_src = IPv4(4,6,7,8),
+				.ip_dst = RTE_IPV4(2,2,3,4),
+				.ip_src = RTE_IPV4(4,6,7,8),
 				.proto = 0x5,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -626,8 +597,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.deny = 23
 		}, /* should match 13, 23 */
 		{
-				.ip_dst = IPv4(1,2,3,4),
-				.ip_src = IPv4(4,6,7,8),
+				.ip_dst = RTE_IPV4(1,2,3,4),
+				.ip_src = RTE_IPV4(4,6,7,8),
 				.proto = 0x5,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -639,8 +610,8 @@ struct ipv4_7tuple acl_test_data[] = {
 
 /* visual separator! */
 		{
-				.ip_dst = IPv4(1,2,100,200),
-				.ip_src = IPv4(5,6,7,254),
+				.ip_dst = RTE_IPV4(1,2,100,200),
+				.ip_src = RTE_IPV4(5,6,7,254),
 				.proto = 0x55,
 				.vlan = 0x8000,
 				.domain = 0x6464,
@@ -649,8 +620,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.allow = 10
 		}, /* should match 10 */
 		{
-				.ip_dst = IPv4(5,6,7,254),
-				.ip_src = IPv4(1,2,100,200),
+				.ip_dst = RTE_IPV4(5,6,7,254),
+				.ip_src = RTE_IPV4(1,2,100,200),
 				.proto = 0x55,
 				.vlan = 0x8100,
 				.domain = 0x6464,
@@ -659,8 +630,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.allow = 10
 		}, /* should match 10 */
 		{
-				.ip_dst = IPv4(1,10,20,30),
-				.ip_src = IPv4(5,6,7,8),
+				.ip_dst = RTE_IPV4(1,10,20,30),
+				.ip_src = RTE_IPV4(5,6,7,8),
 				.proto = 0x55,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -668,8 +639,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.allow = 7
 		}, /* should match 7 */
 		{
-				.ip_dst = IPv4(5,6,7,8),
-				.ip_src = IPv4(1,10,20,30),
+				.ip_dst = RTE_IPV4(5,6,7,8),
+				.ip_src = RTE_IPV4(1,10,20,30),
 				.proto = 0x55,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -677,8 +648,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.allow = 7
 		}, /* should match 7 */
 		{
-				.ip_dst = IPv4(2,2,3,4),
-				.ip_src = IPv4(4,6,7,8),
+				.ip_dst = RTE_IPV4(2,2,3,4),
+				.ip_src = RTE_IPV4(4,6,7,8),
 				.proto = 0x55,
 				.vlan = 0x64,
 				.port_src = 12345,
@@ -686,8 +657,8 @@ struct ipv4_7tuple acl_test_data[] = {
 				.allow = 7
 		}, /* should match 7 */
 		{
-				.ip_dst = IPv4(1,2,3,4),
-				.ip_src = IPv4(4,6,7,8),
+				.ip_dst = RTE_IPV4(1,2,3,4),
+				.ip_src = RTE_IPV4(4,6,7,8),
 				.proto = 0x50,
 				.vlan = 0x6466,
 				.port_src = 12345,
