@@ -255,7 +255,7 @@ const char * ipaddr_invalid_strs[] = {
 		/** misc **/
 
 		/* too long */
-		"1234:1234:1234:1234:1234:1234:1234:1234:1234:1234:1234"
+		"1234:1234:1234:1234:1234:1234:1234:1234:1234:1234:1234",
 		"random invalid text",
 		"",
 		"\0",
