@@ -8,7 +8,6 @@
  *
  */
 
-#define _GNU_SOURCE
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
@@ -26,6 +25,7 @@
 #define ADDR_SIZE 32
 #endif
 
+static int use_stdin, use_stdout;
 
 static const char *sym_name(struct elf_info *elf, Elf_Sym *sym)
 {
@@ -39,11 +39,36 @@ static void *grab_file(const char *filename, unsigned long *size)
 {
 	struct stat st;
 	void *map = MAP_FAILED;
-	int fd;
+	int fd = -1;
+
+	if (!use_stdin) {
+		fd = open(filename, O_RDONLY);
+		if (fd < 0)
+			return NULL;
+	} else {
+		/* from stdin, use a temporary file to mmap */
+		FILE *infile;
+		char buffer[1024];
+		int n;
+
+		infile = tmpfile();
+		if (infile == NULL) {
+			perror("tmpfile");
+			return NULL;
+		}
+		fd = dup(fileno(infile));
+		fclose(infile);
+		if (fd < 0)
+			return NULL;
+
+		n = read(STDIN_FILENO, buffer, sizeof(buffer));
+		while (n > 0) {
+			if (write(fd, buffer, n) != n)
+				goto failed;
+			n = read(STDIN_FILENO, buffer, sizeof(buffer));
+		}
+	}
 
-	fd = open(filename, O_RDONLY);
-	if (fd < 0)
-		return NULL;
 	if (fstat(fd, &st))
 		goto failed;
 
@@ -358,10 +383,14 @@ static void output_pmd_info_string(struct elf_info *info, char *outfile)
 	struct rte_pci_id *pci_ids;
 	int idx = 0;
 
-	ofd = fopen(outfile, "w+");
-	if (!ofd) {
-		fprintf(stderr, "Unable to open output file\n");
-		return;
+	if (use_stdout)
+		ofd = stdout;
+	else {
+		ofd = fopen(outfile, "w+");
+		if (!ofd) {
+			fprintf(stderr, "Unable to open output file\n");
+			return;
+		}
 	}
 
 	drv = info->drivers;
@@ -402,7 +431,7 @@ static void output_pmd_info_string(struct elf_info *info, char *outfile)
 
 int main(int argc, char **argv)
 {
-	struct elf_info info = {0};
+	struct elf_info info;
 	int rc = 1;
 
 	if (argc < 3) {
@@ -411,6 +440,9 @@ int main(int argc, char **argv)
 			basename(argv[0]));
 		exit(127);
 	}
+	memset(&info, 0, sizeof(struct elf_info));
+	use_stdin = !strcmp(argv[1], "-");
+	use_stdout = !strcmp(argv[2], "-");
 	parse_elf(&info, argv[1]);
 
 	if (locate_pmd_entries(&info) < 0)
