@@ -38,12 +38,12 @@ sw_port_link(struct rte_eventdev *dev, void *port, const uint8_t queues[],
 
 		/* check for qid map overflow */
 		if (q->cq_num_mapped_cqs >= RTE_DIM(q->cq_map)) {
-			rte_errno = -EDQUOT;
+			rte_errno = EDQUOT;
 			break;
 		}
 
 		if (p->is_directed && p->num_qids_mapped > 0) {
-			rte_errno = -EDQUOT;
+			rte_errno = EDQUOT;
 			break;
 		}
 
@@ -59,12 +59,12 @@ sw_port_link(struct rte_eventdev *dev, void *port, const uint8_t queues[],
 		if (q->type == SW_SCHED_TYPE_DIRECT) {
 			/* check directed qids only map to one port */
 			if (p->num_qids_mapped > 0) {
-				rte_errno = -EDQUOT;
+				rte_errno = EDQUOT;
 				break;
 			}
 			/* check port only takes a directed flow */
 			if (num > 1) {
-				rte_errno = -EDQUOT;
+				rte_errno = EDQUOT;
 				break;
 			}
 
@@ -717,7 +717,6 @@ sw_dump(struct rte_eventdev *dev, FILE *f)
 			continue;
 		}
 		int affinities_per_port[SW_PORTS_MAX] = {0};
-		uint32_t inflights = 0;
 
 		fprintf(f, "  Queue %d (%s)\n", i, q_type_strings[qid->type]);
 		fprintf(f, "\trx   %"PRIu64"\tdrop %"PRIu64"\ttx   %"PRIu64"\n",
@@ -738,7 +737,6 @@ sw_dump(struct rte_eventdev *dev, FILE *f)
 		for (flow = 0; flow < RTE_DIM(qid->fids); flow++)
 			if (qid->fids[flow].cq != -1) {
 				affinities_per_port[qid->fids[flow].cq]++;
-				inflights += qid->fids[flow].pcount;
 			}
 
 		uint32_t port;
