@@ -216,17 +216,11 @@ mlx5_dev_rss_reta_update(struct rte_eth_dev *dev,
 	for (idx = 0, i = 0; (i != reta_size); ++i) {
 		idx = i / RTE_RETA_GROUP_SIZE;
 		pos = i % RTE_RETA_GROUP_SIZE;
-		if (((reta_conf[idx].mask >> i) & 0x1) == 0)
+		if (((reta_conf[idx].mask >> pos) & 0x1) == 0)
 			continue;
 		assert(reta_conf[idx].reta[pos] < priv->rxqs_n);
 		(*priv->reta_idx)[i] = reta_conf[idx].reta[pos];
 	}
-
 	priv->skip_default_rss_reta = 1;
-
-	if (dev->data->dev_started) {
-		mlx5_dev_stop(dev);
-		return mlx5_dev_start(dev);
-	}
-	return 0;
+	return mlx5_traffic_restart(dev);
 }
