@@ -1294,6 +1294,9 @@ sfc_flow_parse_rss(struct sfc_adapter *sa,
 			rxq_hw_index_max = rxq->hw_index;
 	}
 
+	if (rxq_hw_index_max - rxq_hw_index_min + 1 > EFX_MAXRSS)
+		return -EINVAL;
+
 	switch (action_rss->func) {
 	case RTE_ETH_HASH_FUNCTION_DEFAULT:
 	case RTE_ETH_HASH_FUNCTION_TOEPLITZ:
@@ -1423,9 +1426,8 @@ sfc_flow_filter_insert(struct sfc_adapter *sa,
 		uint8_t *rss_key;
 
 		if (flow->rss) {
-			rss_spread = MIN(flow_rss->rxq_hw_index_max -
-					flow_rss->rxq_hw_index_min + 1,
-					EFX_MAXRSS);
+			rss_spread = flow_rss->rxq_hw_index_max -
+				     flow_rss->rxq_hw_index_min + 1;
 			rss_hash_types = flow_rss->rss_hash_types;
 			rss_key = flow_rss->rss_key;
 		} else {
