@@ -1624,12 +1624,12 @@ flow_rule_action_get(struct pmd_internals *softnic,
 
 			/* RTE_TABLE_ACTION_METER */
 			rule_action->mtr.mtr[0].meter_profile_id = meter_profile_id;
-			rule_action->mtr.mtr[0].policer[e_RTE_METER_GREEN] =
-				softnic_table_action_policer(m->params.action[RTE_MTR_GREEN]);
-			rule_action->mtr.mtr[0].policer[e_RTE_METER_YELLOW] =
-				softnic_table_action_policer(m->params.action[RTE_MTR_YELLOW]);
-			rule_action->mtr.mtr[0].policer[e_RTE_METER_RED] =
-				softnic_table_action_policer(m->params.action[RTE_MTR_RED]);
+			rule_action->mtr.mtr[0].policer[RTE_COLOR_GREEN] =
+				softnic_table_action_policer(m->params.action[RTE_COLOR_GREEN]);
+			rule_action->mtr.mtr[0].policer[RTE_COLOR_YELLOW] =
+				softnic_table_action_policer(m->params.action[RTE_COLOR_YELLOW]);
+			rule_action->mtr.mtr[0].policer[RTE_COLOR_RED] =
+				softnic_table_action_policer(m->params.action[RTE_COLOR_RED]);
 			rule_action->mtr.tc_mask = 1;
 			rule_action->action_mask |= 1 << RTE_TABLE_ACTION_MTR;
 			break;
@@ -1681,9 +1681,9 @@ flow_rule_action_get(struct pmd_internals *softnic,
 					item,
 					"VXLAN ENCAP: first encap item should be ether");
 			}
-			ether_addr_copy(&spec.eth.dst,
+			rte_ether_addr_copy(&spec.eth.dst,
 					&rule_action->encap.vxlan.ether.da);
-			ether_addr_copy(&spec.eth.src,
+			rte_ether_addr_copy(&spec.eth.src,
 					&rule_action->encap.vxlan.ether.sa);
 
 			item++;
