@@ -118,6 +118,10 @@ cryptodev_fips_validate_app_int(void)
 	if (ret < 0)
 		goto error_exit;
 
+	ret = rte_cryptodev_start(env.dev_id);
+	if (ret < 0)
+		goto error_exit;
+
 	return 0;
 
 error_exit:
@@ -433,6 +437,9 @@ exit:
 	fips_test_clear();
 	cryptodev_fips_validate_app_uninit();
 
+	/* clean up the EAL */
+	rte_eal_cleanup();
+
 	return ret;
 
 }
@@ -1030,10 +1037,12 @@ fips_mct_tdes_test(void)
 	int test_mode = info.interim_info.tdes_data.test_mode;
 
 	for (i = 0; i < TDES_EXTERN_ITER; i++) {
-		if (i == 0) {
+		if ((i == 0) && (info.version == 21.4f)) {
 			if (!(strstr(info.vec[0], "COUNT")))
 				fprintf(info.fp_wr, "%s%u\n", "COUNT = ", 0);
-		} else
+		}
+
+		if (i != 0)
 			update_info_vec(i);
 
 		fips_test_write_one_case();
@@ -1368,7 +1377,6 @@ fips_mct_sha_test(void)
 	int ret;
 	uint32_t i, j;
 
-	val.val = rte_malloc(NULL, (MAX_DIGEST_SIZE*SHA_MD_BLOCK), 0);
 	for (i = 0; i < SHA_MD_BLOCK; i++)
 		md[i].val = rte_malloc(NULL, (MAX_DIGEST_SIZE*2), 0);
 
