@@ -475,5 +475,8 @@ main(int argc, char **argv)
 
 	rte_eal_mp_remote_launch(lcore_main, NULL, CALL_MASTER);
 
+	/* clean up the EAL */
+	rte_eal_cleanup();
+
 	return 0;
 }
