@@ -710,5 +710,8 @@ main(int argc, char *argv[])
 	/* call on master too */
 	(void) lcore_main((void*)i);
 
+	/* clean up the EAL */
+	rte_eal_cleanup();
+
 	return 0;
 }
